#include <zephyr/kernel.h>
#include <zephyr/init.h>

/*
 * This file describes application specific configuration of the chip/board.
 */

static int init_example_io(void);
/* NOTE: No kernel services are available in PRE_KERNEL_1. */
/* NOTE: Zephyr ignores the init function return value. */
/* NOTE: "make initlevels" lists all init functions */
SYS_INIT(init_example_io, PRE_KERNEL_1, 10);

#if defined(CONFIG_SOC_GR716A)
#include <drv/clkgate.h>
#include <drv/gr716/clkgate.h>
#include <drv/regs/clkgate_bits.h>
#include <chip/gr716a/pin.h>

static int init_example_io(void) {
  /* clock enable memory controller */
  {
    struct clkgate_priv *clkgate;

    if (0 == clkgate_dev_count()) {
      clkgate_init(GR716_CLKGATE_DRV_ALL);
    }
    clkgate = clkgate_open(0);
    /* reset ftmctrl so we can depend on register reset values */
    clkgate_gate(clkgate, CLKGATE0_GR716_MCTRL0);
    clkgate_enable(clkgate, CLKGATE0_GR716_MCTRL0);
    clkgate_close(clkgate);
  }

  /* configure memory controller for SRAM */
  {
    volatile uint32_t *regs = (void *) 0x80000000;
    /* wait state: 0, bank size: 8 KiB * 2**8 = 2 MiB */
    regs[1] = 0x8u << 9; /* rambanksz in mcfg2 */
    /* disable EDAC */
    regs[2] = 0; /* mcfg3 */
  }

  /* Set pin functions for SRAM */
  {
    for (int i = 0; i <= 18; i++) {
      gr716a_set_pinfunc(i, IO_MODE_MEM); /* A0..A18 */
    }
    gr716a_set_pinfunc(49, IO_MODE_MEM); /* A19 */
    gr716a_set_pinfunc(50, IO_MODE_MEM); /* A20 */
    gr716a_set_pinfunc(19, IO_MODE_MEM); /* RAMSN */
    gr716a_set_pinfunc(34, IO_MODE_MEM); /* WRITEN */
    gr716a_set_pinfunc(33, IO_MODE_MEM); /* OEN */
    for (int i = 25; i <= 32; i++) {
      gr716a_set_pinfunc(i, IO_MODE_MEM); /* D0..D7 */
    }
  }

  return 0;
}
#else
 #error "Don't know how to init pin mux, PLL, clock gating, etc."
#endif

