#include <zephyr/kernel.h>
#include <zephyr/init.h>

/*
 * This file describes application specific configuration of the chip/board.
 */

static int init_example_io(void);
/* NOTE: No kernel services are available in PRE_KERNEL_1. */
/* NOTE: Zephyr ignores the init function return value. */
/* NOTE: "make initlevels" lists all init functions */
SYS_INIT(init_example_io, PRE_KERNEL_1, 10);

#if defined(CONFIG_SOC_GR716A)
#include <drv/clkgate.h>
#include <drv/gr716/clkgate.h>
#include <drv/regs/clkgate_bits.h>
#include <chip/gr716a/pin.h>
#include <chip/gr716a/pll.h>

static void enable_the_grspw2_clock(void) {
  const uint32_t GATEMASK = CLKGATE1_GR716_GRSPW;
  struct clkgate_priv *dev;

  if (0 == clkgate_dev_count()) {
    clkgate_init(GR716_CLKGATE_DRV_ALL);
  }
  dev = clkgate_open(1);
  clkgate_gate(dev, GATEMASK);
  clkgate_enable(dev, GATEMASK);
  clkgate_close(dev);
}

static void enable_the_grspw2_pins(void) {
  gr716a_set_pinfunc(21, IO_MODE_SPW);
  gr716a_set_pinfunc(22, IO_MODE_SPW);
  gr716a_set_pinfunc(23, IO_MODE_SPW);
  gr716a_set_pinfunc(24, IO_MODE_SPW);
}

static int init_example_io(void) {
  gr716a_pll_config(PLL_REF_SPW_CLK, PLL_FREQ_12MHZ, PLL_POWER_ENABLE);
  gr716a_spwclk(SPWCLK_SOURCE_PLL, 2, 0);
  gr716a_set_lvdsfunc(LVDS_MODE_SPW);
  if (0) {
    /* for GRSPW2 port 1 */
    enable_the_grspw2_pins();
  }
  enable_the_grspw2_clock();
  if (1) {
    /* 10 MHz startup and run clock */
    volatile uint32_t *const thereg = (void *) 0x8010000c;
    *thereg = 0x0909;
  }
  return 0;
}

#elif defined(CONFIG_SOC_GR716B)
#include <drv/grspwrouter.h>
static int init_example_io(void) {
  return 0;
}

static struct grspwrouter_devcfg thedevcfg = {
  .regs = {
    .addr       = 0xfff10000u,
    .interrupt  = 5,
    .device_id  = 0x03e,
    .version    = 0,
  },
};

static int init_example_router(void) {
  enum drvret ret;

  grspwrouter_dev *dev;
  printk("router_register()\n");
  grspwrouter_register(&thedevcfg);
  dev = grspwrouter_open(0, 0,
   CONFIG_SYS_CLOCK_HW_CYCLES_PER_SEC / (1000 * 1000));
  /* Route all packets with logical address 0x40 to GRSPW0 */
  ret = grspwrouter_route_set(dev, 0x40, 0x8, 0, 1, 0);
  return 0;
}
SYS_INIT(init_example_router, PRE_KERNEL_2, 10);

#else
 #error "Don't know how to init pin mux, PLL, clock gating, etc."
#endif

