#include <zephyr/kernel.h>
#include <drv/osal.h>

#define DT_DRV_COMPAT gaisler_irqmp

/* TODO: irqmp map availability should be probed */
static int osal_isr_translate(int busintline) {
  volatile uint32_t *regs;
  volatile uint32_t *mapreg;
  int irqmpintline;
  int offset;
  int index;
  uint32_t map;

  regs = (volatile uint32_t *) DT_INST_REG_ADDR(0);
  mapreg = &regs[0x300/4];
  offset = busintline >> 2;
  index = ((~busintline) & 0x3) * 8;
  map = mapreg[offset];
  irqmpintline = (map >> index) & 0xff;
  return irqmpintline;
}

int osal_isr_register(
  struct osal_isr_ctx *ctx,
  int source,
  osal_isr_t handler,
  void *arg
) {
  int irqmpsrc;

  irqmpsrc = osal_isr_translate(source);
  if ((irqmpsrc < 0) || (31 < irqmpsrc)) {
    return DRV_FAIL;
  }
  irq_connect_dynamic(irqmpsrc, 0, (void (*)(const void *)) handler, arg, 0);
  irq_enable(irqmpsrc);
  return DRV_OK;
}

int osal_isr_unregister(
  struct osal_isr_ctx *ctx,
  int source,
  osal_isr_t handler,
  void *arg
) {
  int ret;
  int irqmpsrc;

  irqmpsrc = osal_isr_translate(source);
  /* TODO: Should disable only on last unregister call */
  irq_disable(irqmpsrc);
  ret = irq_disconnect_dynamic(irqmpsrc, 0,
   (void (*)(const void *)) handler, arg, 0);
  if (ret != 0) {
    return DRV_FAIL;
  }
  return DRV_OK;
}

/*
 * Zephyr is currently single-processor on GRLIB systems. Implement this with a
 * spinlock anyways for future compatibility.
 */
static struct k_spinlock osal_ldstub_spinlock; /* zero init */
uint8_t osal_ldstub(uint8_t *addr) {
  uint8_t tmp;
  k_spinlock_key_t key;

  key = k_spin_lock(&osal_ldstub_spinlock);
  {
    tmp = *addr;
    *addr = 0xff;
  }
  k_spin_unlock(&osal_ldstub_spinlock, key);

  return tmp;
}

