/* SPDX-License-Identifier: GPL-3.0-or-later */

/*
 * Copyright 2023 Cobham Gaisler AB
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <bcc/ambapp.h>
#include <drv/ambapp_ids.h>
#include <drv/grspwrouter.h>
#include <drv/nelem.h>
#include <drv/auto.h>

static uint32_t fn(
        void *info,
        uint32_t vendor,
        uint32_t device,
        uint32_t type,
        uint32_t depth,
        void *arg
)
{
        UNUSED(vendor); UNUSED(device); UNUSED(type); UNUSED(depth);
        int *count = arg;
        struct amba_ahb_info *ahbi = info;
        struct amba_ahb_bar *bar = &ahbi->bar[0];

        struct drv_devreg *node;

        node = malloc(sizeof (struct grspwrouter_devcfg));
        if (!node) {
                return 1;
        }
        node->addr = bar->start;
        node->interrupt = ahbi->irq;
        node->device_id = device;
        node->version = ahbi->ver;
        grspwrouter_register((struct grspwrouter_devcfg *) node);

        (*count)++;

        return 0;
}

int grspwrouter_autoinit(void)
{
        static const uint32_t ioarea = 0xfff00000;
        int count = 0;

        ambapp_visit(
                ioarea,
                VENDOR_GAISLER,
                GAISLER_SPWROUTER,
                AMBAPP_VISIT_AHBSLAVE,
                4,
                fn,
                &count
        );
        return count;
}
