/* SPDX-License-Identifier: GPL-3.0-or-later */

/*
 * Copyright 2023 Cobham Gaisler AB
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <drv/grspwrouter.h>
#include "priv.h"

static uint32_t dev_count;
static struct drv_list devlist = { NULL, NULL };

drvret grspwrouter_register(struct grspwrouter_devcfg *devcfg)
{
    drv_list_addtail(&devlist, &devcfg->regs.node);
    dev_count++;
    return DRV_OK;
}

uint32_t grspwrouter_dev_count(void)
{
    return dev_count;
}

grspwrouter_dev * grspwrouter_open(uint32_t dev_no, uint32_t spw_freq, int32_t sys_freq)
{
    uint32_t rtrcfg = 0;
    grspwrouter_devcfg * devcfg;
    grspwrouter_dev * dev;
    uint8_t popen;
    drvret ret;

    if (dev_no >= dev_count)
        return NULL;

    devcfg = (grspwrouter_devcfg *) drv_list_getbyindex(&devlist, dev_no);
    dev = &devcfg->dev;

    popen = osal_ldstub(&dev->open);
    if (popen) {
        return NULL;
    }

    dev->regs = (grspwrouter_regs *) devcfg->regs.addr;
    dev->interrupt = devcfg->regs.interrupt;
    dev->spw_freq = spw_freq;

    ret = osal_isr_register(
            &dev->isr_ctx,
            dev->interrupt,
            (osal_isr_t)grspwrouter_isr,
            dev
    );

    if (ret != DRV_OK) {
            dev->open = 0;
            return NULL;
    }

    if (spw_freq != 0) {
        REG_WRITE(&dev->regs->idiv, (spw_freq / 10) - 1);
    } else {
        dev->spw_freq = (REG_READ(&dev->regs->idiv) + 1) * 10;
    }

    if (sys_freq != 0)
        REG_WRITE(&dev->regs->prescaler, sys_freq * 100 - 1);

    rtrcfg = REG_READ(&dev->regs->rtrcfg);

    dev->max_spw_ports = GRSPWROUTER_RTRCFG_SP(rtrcfg) + 1;
    dev->max_ports = GRSPWROUTER_RTRCFG_SP(rtrcfg) + GRSPWROUTER_RTRCFG_AP(rtrcfg) + 1;

    return dev;
}
