/* SPDX-License-Identifier: GPL-3.0-or-later */

/*
 * Copyright 2023 Cobham Gaisler AB
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <drv/grspwrouter.h>
#include "priv.h"

drvret grspwrouter_ic_enable(grspwrouter_dev *dev, uint32_t timeout, uint32_t cooldown)
{
    uint32_t rtrcfg;
    uint32_t isrtimer;

    if ((cooldown / 100) & ~0x1f)
        return DRV_INVAL;

    rtrcfg = REG_READ(&dev->regs->rtrcfg);
    GRSPWROUTER_RTRCFG_IE_SET(rtrcfg, 0);
    REG_WRITE(&dev->regs->rtrcfg, rtrcfg);

    REG_WRITE(&dev->regs->isrtimer, timeout / 100);
    isrtimer = REG_READ(&dev->regs->isrtimer);
    if (isrtimer != timeout / 100)
        return DRV_INVAL;

    REG_WRITE(&dev->regs->isrctimer, cooldown / 100);

    GRSPWROUTER_RTRCFG_IE_SET(rtrcfg, 1);
    GRSPWROUTER_RTRCFG_IC_SET(rtrcfg, cooldown != 0 ? 1 : 0);
    REG_WRITE(&dev->regs->rtrcfg, rtrcfg);

    return DRV_OK;
}
