/* SPDX-License-Identifier: GPL-3.0-or-later */

/*
 * Copyright 2023 Cobham Gaisler AB
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <drv/grspwrouter.h>
#include "priv.h"

#define ERROR_MASK (GRSPWROUTER_STATUS_ERR_TRUNC |\
                   GRSPWROUTER_STATUS_ERR_INTTRUNC |\
                   GRSPWROUTER_STATUS_ERR_RMAP |\
                   GRSPWROUTER_STATUS_ERR_NOTRDY |\
                   GRSPWROUTER_STATUS_ERR_TIMEOUT |\
                   GRSPWROUTER_STATUS_ERR_MEM |\
                   GRSPWROUTER_STATUS_ERR_ADRS |\
                   GRSPWROUTER_STATUS_ERR_CREDIT |\
                   GRSPWROUTER_STATUS_ERR_ESCAPE |\
                   GRSPWROUTER_STATUS_ERR_DISCON |\
                   GRSPWROUTER_STATUS_ERR_PARITY)

void grspwrouter_isr(grspwrouter_dev *dev)
{
    uint32_t i;
    uint32_t psts;
    uint32_t pip;

    if (dev->isr == 0)
        return;

    pip = REG_READ(&dev->regs->pip);

    for (i = 0; i < dev->max_ports && pip; i++) {
        if (pip & (1 << i)) {
            psts = REG_READ(&dev->regs->psts[i]);
            dev->isr(i, GRSPWROUTER_PSTS_LS(psts), psts & ERROR_MASK, dev->isr_arg);
            REG_WRITE(&dev->regs->psts[i], psts);
            REG_WRITE(&dev->regs->pip, (1 << i));
        }
    }
}
