/* SPDX-License-Identifier: GPL-3.0-or-later */

/*
 * Copyright 2023 Cobham Gaisler AB
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <drv/grspwrouter.h>
#include "priv.h"

drvret grspwrouter_port_link_start(grspwrouter_dev * dev, uint8_t port, uint32_t link_rate)
{
    uint32_t pctrl;
    uint32_t rdiv;

    if (port == 0 || port >= dev->max_spw_ports)
        return DRV_INVAL;

    if (link_rate == 0)
        return DRV_INVAL;

    rdiv = (dev->spw_freq / link_rate) - 1;

    if (rdiv & ~0xff)
        return DRV_INVAL;

    pctrl = REG_READ(&dev->regs->pctrl[port]);
    GRSPWROUTER_PCTRL_RD_SET(pctrl, rdiv);
    GRSPWROUTER_PCTRL_LD_SET(pctrl, 0);
    GRSPWROUTER_PCTRL_LS_SET(pctrl, 1);
    REG_WRITE(&dev->regs->pctrl[port], pctrl);

    return DRV_OK;
}
