/* SPDX-License-Identifier: GPL-3.0-or-later */

/*
 * Copyright 2023 Cobham Gaisler AB
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <drv/grspwrouter.h>
#include "priv.h"

drvret grspwrouter_port_timeout(grspwrouter_dev * dev, uint8_t port, uint32_t timeout, bool overrun, bool underrun, bool autodisconnect)
{
    uint32_t pctrl;
    uint32_t pctrl2;
    uint32_t ptimer;

    if (port == 0 || port >= dev->max_ports)
        return DRV_INVAL;

    if (autodisconnect && port >= dev->max_spw_ports)
        return DRV_INVAL;

    pctrl = REG_READ(&dev->regs->pctrl[port]);
    pctrl2 = REG_READ(&dev->regs->pctrl2[port]);

    GRSPWROUTER_PCTRL_AD_SET(pctrl, autodisconnect ? 1 : 0);
    GRSPWROUTER_PCTRL_TR_SET(pctrl, 1);
    GRSPWROUTER_PCTRL2_OR_SET(pctrl2, overrun ? 1 : 0);
    GRSPWROUTER_PCTRL2_UR_SET(pctrl2, underrun ? 1 : 0);

    REG_WRITE(&dev->regs->ptimer[port], timeout / 100);
    ptimer = REG_READ(&dev->regs->ptimer[port]);
    if (ptimer != timeout / 100)
        return DRV_INVAL;

    REG_WRITE(&dev->regs->pctrl2[port], pctrl2);
    REG_WRITE(&dev->regs->pctrl[port], pctrl);

    return DRV_OK;
}
