/* SPDX-License-Identifier: GPL-3.0-or-later */

/*
 * Copyright 2023 Cobham Gaisler AB
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <drv/grspwrouter.h>
#include "priv.h"

drvret grspwrouter_route_set(grspwrouter_dev *dev, uint8_t address, uint32_t to_mask, bool header_deletion, bool spill_packet, uint32_t options)
{
    uint32_t ctrl = 0;
    uint32_t map = to_mask;

    if (address == 0)
        return DRV_INVAL;

    GRSPWROUTER_RTPMAP_PD_SET(map, options & GRSPWROUTER_PACKET_DIST ? 1 : 0);

    REG_WRITE(&dev->regs->rtpmap[address], map);

    GRSPWROUTER_RTACTRL_EN_SET(ctrl, 1);
    if (spill_packet)
        GRSPWROUTER_RTACTRL_SR_SET(ctrl, 1);
    if (options & GRSPWROUTER_ROUTE_PRIORITY)
        GRSPWROUTER_RTACTRL_PR_SET(ctrl, 1);
    if (header_deletion)
        GRSPWROUTER_RTACTRL_HD_SET(ctrl, 1);

    REG_WRITE(&dev->regs->rtactrl[address], ctrl);

    return DRV_OK;
}
