/*
 * Copyright (c) 2023, Cobham Gaisler AB
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Register definitions for GRLIB NANDFCTRL2
 * Author: Martin Åberg
 */
#include <stdint.h>

struct nf2_regs {
  uint32_t ctrl;
  uint32_t inten;
  uint32_t cmd;
  uint32_t perm;
  uint32_t sefi;
  uint32_t x014;
  uint32_t x018;
  uint32_t x01c;
  uint32_t stat0;
  uint32_t stat1;
  uint32_t stat2;
  uint32_t stat3;
  uint32_t x030;
  uint32_t x034;
  uint32_t cap[5];      /* 038..048 */
  uint32_t x04c;
  uint32_t tme[6];      /* 050..064 */
  uint32_t x068;
  uint32_t x06c;

  uint32_t x070_0cf[(6 * 0x10) / 4];

  uint32_t timeout0;    /* 0d0 */
  uint32_t x0d4;
  uint32_t x0d8;
  uint32_t x0dc;
  uint32_t x0e0_14f[(0x150-0x0e0) / 4];
  uint32_t llpl;        /* 150 */
  uint32_t llph;        /* 154 */
  uint32_t x158;
  uint32_t x15c;
  uint32_t dcmd;        /* 160 */
  uint32_t dtarsel[2];  /* 164..168 */
  uint32_t dchsel;      /* 16c */
  uint32_t drbsel;      /* 170 */
  uint32_t drow;
  uint32_t dcol;
  uint32_t dstat;
  uint32_t deccstat;    /* 180 */
  uint32_t x184;
  uint32_t x188;
  uint32_t x18c;
  uint32_t ddpl;        /* 190 */
  uint32_t ddph;
  uint32_t x198;
  uint32_t x19c;
  uint32_t x1ax[4];
  uint32_t x1bx[4];
  uint32_t x1cx[4];
  uint32_t x1dx[4];
  uint32_t datain[4];
  uint32_t dataout[4];
};

#define NF2_CTRL_EE           0x00002000
#define NF2_CTRL_RE           0x00001000
#define NF2_CTRL_LLM          0x00000400
#define NF2_CTRL_BBM          0x00000200
#define NF2_CTRL_EDO          0x00000080
#define NF2_CTRL_MSEL_BIT      3
#define NF2_CTRL_MSEL         (0x3<<3)

#define NF2_INTEN_TMOUT       0x00000040
#define NF2_INTEN_CMD         0x00000020
#define NF2_INTEN_DL          0x00000010
#define NF2_INTEN_UL          0x00000008
#define NF2_INTEN_ECC         0x00000004
#define NF2_INTEN_DS          0x00000002
#define NF2_INTEN_EN          0x00000001

#define NF2_CMD_ABORT         0x00000004
#define NF2_CMD_DT            0x00000002
#define NF2_CMD_RST           0x00000001

#define NF2_STAT0_DA          0x00000008

#define NF2_STAT1_TMOUT       0x00000040
#define NF2_STAT1_CMD         0x00000020
#define NF2_STAT1_DL          0x00000010
#define NF2_STAT1_UL          0x00000008
#define NF2_STAT1_ECC         0x00000004
#define NF2_STAT1_DS          0x00000002
#define NF2_STAT1_EN          0x00000001

#define NF2_CAP0_MEMSUP_BIT   23
#define NF2_CAP0_MEMSUP       (0x03ul << 23)
#define NF2_CAP0_FT_BIT       20
#define NF2_CAP0_FT           (0x07ul << 20)
#define NF2_CAP0_PRBBM        0x00080000
#define NF2_CAP0_RND          0x00040000
#define NF2_CAP0_NECC_BIT     16
#define NF2_CAP0_NECC         (0x03ul << 16)
#define NF2_CAP0_NRB_BIT      11
#define NF2_CAP0_NRB          (0x1ful << 11)
#define NF2_CAP0_NCHAN_BIT     6
#define NF2_CAP0_NCHAN        (0x1ful <<  6)
#define NF2_CAP0_NTARG_BIT     0
#define NF2_CAP0_NTARG        (0x3ful <<  0)

#define NF2_CAP1_E1CAP_BIT    26
#define NF2_CAP1_E1CAP        (0x3ful << 26)
#define NF2_CAP1_E1CHUNK_BIT  21
#define NF2_CAP1_E1CHUNK      (0x1ful << 21)
#define NF2_CAP1_E1GF_BIT     16
#define NF2_CAP1_E1GF         (0x1ful << 16)
#define NF2_CAP1_E0CAP_BIT    10
#define NF2_CAP1_E0CAP        (0x3ful << 10)
#define NF2_CAP1_E0CHUNK_BIT   5
#define NF2_CAP1_E0CHUNK      (0x1ful <<  5)
#define NF2_CAP1_E0GF_BIT      0
#define NF2_CAP1_E0GF         (0x1ful <<  0)

/* CAP2, CAP3, CAP4 have the same layout */
#define NF2_CAP2_M0SEL_BIT    31
#define NF2_CAP2_M0SEL        (1ul << 31)
#define NF2_CAP2_M0SPARE_BIT  16
#define NF2_CAP2_M0SPARE      (0x1ffful << 16)
#define NF2_CAP2_M0DATA_BIT    0
#define NF2_CAP2_M0DATA       (0xfffful << 0)

#define NF2_DCMD_CMD2_BIT     24
#define NF2_DCMD_CMD2         (0xfful << 24)
#define NF2_DCMD_CMD1_BIT     16
#define NF2_DCMD_CMD1         (0xfful << 16)
#define NF2_DCMD_DD           0x00000800
#define NF2_DCMD_ED           0x00000400
#define NF2_DCMD_RD           0x00000200
#define NF2_DCMD_SUB_BIT       7
#define NF2_DCMD_SUB          (0x3 << 7)
#define NF2_DCMD_SRB          0x00000040
#define NF2_DCMD_SC2          0x00000008
#define NF2_DCMD_IE           0x00000002
#define NF2_DCMD_EN           0x00000001

#define NF2_DROW_ADDR         0x00fffffful
#define NF2_DCOL_SIZE         0xffff0000ul
#define NF2_DCOL_ADDR         0x0000fffful

